#!/usr/bin/perl
###############################################################################
#   Copyright (C) 2004 Craig B. Agricola (craig@theagricolas.org)
#
# See bandwidth_monitor.c, which should be found in the same package as this
# file for license and distribution.  Do not redistribute this script without
# the entire tar-ball.  The original tar-ball is most likely available from
# somewhere under http://theagricolas.org/, currently
# http://theagricolas.org/craig/
#
###############################################################################

sub get_tx_rx_byte_counts {
  my ($dev) = @_;
  my (@counts);
  if (open(F, "/proc/net/dev")) {
	while(<F>) {
	  if (/^\s*$dev:\s*(.*)/) {
		@counts = (split(/\s+/, $1))[0,8];
	  }
	}
	close(F);
  }
  return(@counts);
}

sub calculate_bandwidth {
  my ($delay, $start, $end) = @_;
  my ($bw, $bwu);
  $bw = ($end - $start) * 8 / $delay;
  $bwu = 0;
  while ($bw > 1024) { $bw /= 1024; $bwu++; }
  $bwu  = (" ", "K", "M", "G")[$bwu];
  $bwu .= "bps";
  return($bw, $bwu);
}

my ($dev) = shift(@ARGV);
my ($delay) = shift(@ARGV);
my ($count) = shift(@ARGV);
if (!defined($dev))   { $dev   = "eth0"; }
if (!defined($delay)) { $delay = 1;      }
my ($saved_count) = $count;

my ($orx, $otx) = get_tx_rx_byte_counts($dev);
my ($srx, $stx) = ($orx, $otx);
my ($rx, $tx);
while((!defined($count)) || ($count>0)) {
  select(undef, undef, undef, $delay);
  ($rx, $tx) = get_tx_rx_byte_counts($dev);
  printf("Receive: %9.3f %s     Transmit: %9.3f %s\n",
         calculate_bandwidth($delay, $orx, $rx),
         calculate_bandwidth($delay, $otx, $tx));
  ($orx, $otx) = ($rx, $tx);
  if (defined($count)) { $count--; }
}
print "\nOverall Average:\n";
printf("Receive: %9.3f %s     Transmit: %9.3f %s\n",
	   calculate_bandwidth($delay * $saved_count, $srx, $rx),
	   calculate_bandwidth($delay * $saved_count, $stx, $tx));
